@@ -10,8 +10,6 @@
 #include "../internal/pthread_impl.h"
 #include <assert.h>
 #include <dirent.h>
-#include <emscripten.h>
-#include <emscripten/threading.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <math.h>
@@ -30,6 +28,10 @@
 #include <unistd.h>
 #include <utime.h>
 
+#include <emscripten.h>
+#include <emscripten/threading.h>
+#include <emscripten/stack.h>
+
 // With LLVM 3.6, C11 is the default compilation mode.
 // gets() is deprecated under that standard, but emcc
 // still provides it, so always include it in the build.
@@ -152,20 +154,6 @@ void emscripten_thread_sleep(double msecs) {
     EM_THREAD_STATUS_SLEEPING, EM_THREAD_STATUS_RUNNING);
 }
 
-int nanosleep(const struct timespec* req, struct timespec* rem) {
-  if (!req || req->tv_nsec < 0 || req->tv_nsec > 999999999L || req->tv_sec < 0) {
-    errno = EINVAL;
-    return -1;
-  }
-  emscripten_thread_sleep(req->tv_sec * 1000.0 + req->tv_nsec / 1e6);
-  return 0;
-}
-
-int usleep(unsigned usec) {
-  emscripten_thread_sleep(usec / 1e3);
-  return 0;
-}
-
 // Allocator and deallocator for em_queued_call objects.
 static em_queued_call* em_queued_call_malloc() {
   em_queued_call* call = (em_queued_call*)malloc(sizeof(em_queued_call));
@@ -183,7 +171,10 @@ static void em_queued_call_free(em_queued_call* call) {
   free(call);
 }
 
-void emscripten_async_waitable_close(em_queued_call* call) { em_queued_call_free(call); }
+void emscripten_async_waitable_close(em_queued_call* call) {
+  assert(call->operationDone);
+  em_queued_call_free(call);
+}
 
 extern double emscripten_receive_on_main_thread_js(int functionIndex, int numCallArgs, double* args);
 extern int _emscripten_notify_thread_queue(pthread_t targetThreadId, pthread_t mainThreadId);
@@ -350,7 +341,7 @@ static void _do_call(em_queued_call* q) {
   // If the caller is detached from this operation, it is the main thread's responsibility to free
   // up the call object.
   if (q->calleeDelete) {
-    emscripten_async_waitable_close(q);
+    em_queued_call_free(q);
     // No need to wake a listener, nothing is listening to this since the call object is detached.
   } else {
     // The caller owns this call object, it is listening to it and will free it up.
@@ -741,7 +732,7 @@ int emscripten_sync_run_in_main_runtime_thread_(EM_FUNC_SIGNATURE sig, void* fun
   return q.returnValue.i;
 }
 
-EMSCRIPTEN_KEEPALIVE double emscripten_run_in_main_runtime_thread_js(int index, int num_args, double* buffer, int sync) {
+EMSCRIPTEN_KEEPALIVE double emscripten_run_in_main_runtime_thread_js(int index, int num_args, int64_t* buffer, int sync) {
   em_queued_call q;
   em_queued_call *c;
   if (sync) {
@@ -754,16 +745,19 @@ EMSCRIPTEN_KEEPALIVE double emscripten_run_in_main_runtime_thread_js(int index,
   c->calleeDelete = 1-sync;
   c->functionEnum = EM_PROXIED_JS_FUNCTION;
   c->functionPtr = (void*)index;
-  // We write out the JS doubles into args[], which must be of appropriate size - JS will assume that.
-  assert(sizeof(em_variant_val) == sizeof(double));
   assert(num_args+1 <= EM_QUEUED_JS_CALL_MAX_ARGS);
+  // The types are only known at runtime in these calls, so we store values that
+  // must be able to contain any valid JS value, including a 64-bit BigInt if
+  // BigInt support is enabled. We store to an i64, which can contain both a
+  // BigInt and a JS Number which is a 64-bit double.
   c->args[0].i = num_args;
   for (int i = 0; i < num_args; i++) {
-    c->args[i+1].d = buffer[i];
+    c->args[i+1].i64 = buffer[i];
   }
 
   if (sync) {
     emscripten_sync_run_in_main_thread(&q);
+    // TODO: support BigInt return values somehow.
     return q.returnValue.d;
   } else {
     // 'async' runs are fire and forget, where the caller detaches itself from the call object after
@@ -913,6 +907,11 @@ int llvm_atomic_load_add_i32_p0i32(int* ptr, int delta) {
   return emscripten_atomic_add_u32(ptr, delta);
 }
 
+// Stores the memory address that the main thread is waiting on, if any. If
+// the main thread is waiting, we wake it up before waking up any workers.
+EMSCRIPTEN_KEEPALIVE
+void* _emscripten_main_thread_futex;
+
 typedef struct main_args {
   int argc;
   char** argv;
@@ -933,10 +932,9 @@ int proxy_main(int argc, char** argv) {
   if (emscripten_has_threading_support()) {
     pthread_attr_t attr;
     pthread_attr_init(&attr);
-    pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_JOINABLE);
-    // Use TOTAL_STACK for the stack size, which is the normal size of the stack
+    // Use the size of the current stack, which is the normal size of the stack
     // that main() would have without PROXY_TO_PTHREAD.
-    pthread_attr_setstacksize(&attr, EM_ASM_INT({ return TOTAL_STACK }));
+    pthread_attr_setstacksize(&attr, emscripten_stack_get_base() - emscripten_stack_get_end());
     // Pass special ID -1 to the list of transferred canvases to denote that the thread creation
     // should instead take a list of canvases that are specified from the command line with
     // -s OFFSCREENCANVASES_TO_PTHREAD linker flag.
