@@ -10,8 +10,6 @@ v128_t TESTFN i8x16_load(void *ptr) {
   return wasm_v128_load(ptr);
 }
 
-#ifdef __wasm_unimplemented_simd128__
-
 v128_t TESTFN v8x16_load_splat(void *ptr) {
   return wasm_v8x16_load_splat(ptr);
 }
@@ -43,8 +41,6 @@ v128_t TESTFN i64x2_load32x2_u(void *ptr) {
   return wasm_u64x2_load_32x2(ptr);
 }
 
-#endif // __wasm_unimplemented_simd128__
-
 void TESTFN i8x16_store(void *ptr, v128_t vec) {
   wasm_v128_store(ptr, vec);
 }
@@ -106,14 +102,10 @@ v128_t TESTFN i64x2_shuffle_reverse(v128_t vec) {
   return wasm_v64x2_shuffle(vec, vec, 1, 0);
 }
 
-#ifdef __wasm_unimplemented_simd128__
-
 v128_t TESTFN v8x16_swizzle(v128_t x, v128_t y) {
   return wasm_v8x16_swizzle(x, y);
 }
 
-#endif // __wasm_unimplemented_simd128__
-
 v128_t TESTFN i8x16_splat(int32_t x) {
   return wasm_i8x16_splat(x);
 }
@@ -363,14 +355,10 @@ v128_t TESTFN v128_xor(v128_t x, v128_t y) {
   return wasm_v128_xor(x, y);
 }
 
-#ifdef __wasm_unimplemented_simd128__
-
 v128_t TESTFN v128_andnot(v128_t x, v128_t y) {
   return wasm_v128_andnot(x, y);
 }
 
-#endif // __wasm_unimplemented_simd128__
-
 v128_t TESTFN v128_bitselect(v128_t x, v128_t y, v128_t cond) {
   return wasm_v128_bitselect(x, y, cond);
 }
@@ -413,15 +401,6 @@ v128_t TESTFN i8x16_sub_saturate_s(v128_t x, v128_t y) {
 v128_t TESTFN i8x16_sub_saturate_u(v128_t x, v128_t y) {
   return wasm_u8x16_sub_saturate(x, y);
 }
-
-#ifdef __wasm_unimplemented_simd128__
-
-v128_t TESTFN i8x16_mul(v128_t x, v128_t y) {
-  return wasm_i8x16_mul(x, y);
-}
-
-#endif // __wasm_unimplemented_simd128__
-
 v128_t TESTFN i8x16_min_s(v128_t x, v128_t y) {
   return wasm_i8x16_min(x, y);
 }
@@ -810,8 +789,6 @@ int EMSCRIPTEN_KEEPALIVE __attribute__((__optnone__)) main(int argc, char** argv
                i8x16(7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7));
   }
 
-#ifdef __wasm_unimplemented_simd128__
-
   {
     char vec[8] = {0x80, 0x90, 0xa0, 0xb0, 0xc0, 0xd0, 0xe0, 0xf0};
     void *ptr = &vec;
@@ -852,8 +829,6 @@ int EMSCRIPTEN_KEEPALIVE __attribute__((__optnone__)) main(int argc, char** argv
     );
   }
 
-#endif // __wasm_unimplemented_simd128__
-
   expect_vec(i8x16_const(), u8x16(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16));
   expect_vec(i16x8_const(), u16x8(1, 2, 3, 4, 5, 6, 7, 8));
   expect_vec(i32x4_const(), u32x4(1, 2, 3, 4));
@@ -882,8 +857,6 @@ int EMSCRIPTEN_KEEPALIVE __attribute__((__optnone__)) main(int argc, char** argv
   expect_vec(i32x4_shuffle_reverse((v128_t)i32x4(1, 2, 3, 4)), i32x4(4, 3, 2, 1));
   expect_vec(i64x2_shuffle_reverse((v128_t)i64x2(1, 2)), i64x2(2, 1));
 
-#ifdef  __wasm_unimplemented_simd128__
-
   expect_vec(
     v8x16_swizzle(
       (v128_t)i8x16(0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7,
@@ -894,8 +867,6 @@ int EMSCRIPTEN_KEEPALIVE __attribute__((__optnone__)) main(int argc, char** argv
           0x00, 0x00, 0xff, 0xfd, 0xfc, 0xf8, 0xf4, 0xf0)
   );
 
-#endif // __wasm_unimplemented_simd128__
-
   // i8x16 lane accesses
   expect_vec(i8x16_splat(5), i8x16(5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5));
   expect_vec(i8x16_splat(257), i8x16(1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1));
@@ -1275,16 +1246,10 @@ int EMSCRIPTEN_KEEPALIVE __attribute__((__optnone__)) main(int argc, char** argv
     v128_xor((v128_t)i32x4(0, 0, -1, -1), (v128_t)i32x4(0, -1, 0, -1)),
     i32x4(0, -1, -1, 0)
   );
-
-#ifdef __wasm_unimplemented_simd128__
-
   expect_vec(
     v128_andnot((v128_t)i32x4(0, 0, -1, -1), (v128_t)i32x4(0, -1, 0, -1)),
-    i32x4(0, -1, -1, 0)
+    i32x4(0, 0, -1, 0)
   );
-
-#endif // __wasm_unimplemented_simd128__
-
   expect_vec(
     v128_bitselect(
       (v128_t)i32x4(0xAAAAAAAA, 0xAAAAAAAA, 0xAAAAAAAA, 0xAAAAAAAA),
@@ -1377,19 +1342,6 @@ int EMSCRIPTEN_KEEPALIVE __attribute__((__optnone__)) main(int argc, char** argv
     ),
     u8x16(0, 0, 254, 0, 0, 123, 0, 12, 61, 167, 158, 100, 17, 0, 0, 0)
   );
-
-#ifdef __wasm_unimplemented_simd128__
-
-  expect_vec(
-    i8x16_mul(
-      (v128_t)i8x16(0, 42, 255, 128, 127, 129, 6, 29, 103, 196, 231, 142, 17, 250, 1, 73),
-      (v128_t)i8x16(3, 231, 1, 128, 129, 6, 103, 17, 42, 29, 73, 42, 0, 255, 127, 142)
-    ),
-    i8x16(0, 230, 255, 0, 255, 6, 106, 237, 230, 52, 223, 76, 0, 6, 127, 126)
-  );
-
-#endif // __wasm_unimplemented_simd128__
-
   expect_vec(
     i8x16_min_s(
       (v128_t)i8x16(0, 42, 255, 128, 127, 129, 6, 29, 103, 196, 231, 142, 17, 250, 1, 73),
